unit DataMain;

interface

uses
  SysUtils, Classes, DBXpress, FMTBcd, DB, SqlExpr, Provider, DBClient;

type
  TdtmdlMain = class(TDataModule)
    SQLConnection1: TSQLConnection;
    cdsMembers: TClientDataSet;
    dspMembers: TDataSetProvider;
    SQLMembers: TSQLQuery;
    cdsAttendance: TClientDataSet;
    dspAttendance: TDataSetProvider;
    qryAttendance: TSQLQuery;
    dsMembers: TDataSource;
    dsAttendance: TDataSource;
    cdsClubInfo: TClientDataSet;
    cdsClubInfoNextNumber: TIntegerField;
    cdsClubInfoDatabase: TStringField;
    cdsClubInfoClubCode: TStringField;
    dsClubInfo: TDataSource;
    procedure cdsApplyUpdates(DataSet: TDataSet);
    procedure SetNewRecordDefault(DataSet: TDataSet);
    procedure DataModuleCreate(Sender: TObject);
    procedure SQLConnection1AfterConnect(Sender: TObject);
    procedure SQLConnection1BeforeConnect(Sender: TObject);
    procedure SaveClubInfo(DataSet: TDataSet);
    procedure cdsReconcileError(DataSet: TCustomClientDataSet;
      E: EReconcileError; UpdateKind: TUpdateKind;
      var Action: TReconcileAction);
    procedure dsMembersDataChange(Sender: TObject; Field: TField);
    procedure DataModuleDestroy(Sender: TObject);
  private
    function GetUID(DataSet: TDataSet): string;
    { Private declarations }
  public
    ClubCode : string;
    { Public declarations }
  end;

const
  tcAttendance : string = 'ATND';
  tcMembers    : string = 'MBRS';

var
  dtmdlMain: TdtmdlMain;

implementation

{$R *.dfm}

uses Forms, Dialogs, formReconsile;

procedure TdtmdlMain.SQLConnection1AfterConnect(Sender: TObject);
begin
  // Open Club 1 after connection
  cdsMembers.Open;
  cdsAttendance.Open;
end;

procedure TdtmdlMain.cdsApplyUpdates(DataSet: TDataSet);
begin
  try
    if DataSet is TClientDataSet then
      TClientDataSet(DataSet).ApplyUpdates(-1);
  except
    on e:exception do begin
      ShowMessage('Error Applying updates to'+' '+DataSet.Name+#13+e.message)
    end
  end;
end;

procedure TdtmdlMain.SetNewRecordDefault(DataSet: TDataSet);
begin
  // To easy the demo, Each CDS has a seperate tag to keep the code togeather.
  if Assigned(DataSet.FindField('UID')) then begin
    DataSet.FieldByName('UID').AsString := GetUID(DataSet);
  end;

  if Assigned(DataSet.FindField('CLUB_CODE')) then begin
    DataSet.FieldByName('CLUB_CODE').AsString := ClubCode;
  end;   
end;

function TdtmdlMain.GetUID(DataSet: TDataSet): string;
begin
  // Typically you could have a global list of table id's and pass those in with the site code to
  // Get the next number form the database.
  // I'd normally do this with a trigger in the datbase. but you get the idea.
  cdsClubInfo.Edit;
  cdsClubInfo.FieldByName('NEXT_NUMBER').AsInteger := cdsClubInfo.FieldByName('NEXT_NUMBER').AsInteger +1;
  cdsClubInfo.Post;

  Result := IntToStr(cdsClubInfo.FieldByName('NEXT_NUMBER').AsInteger);

  while Length(Result) < 10 do
    Result := '0'+Result;

  if DataSet.Name = 'cdsMembers' then
    Result := ClubCode +tcMembers +Result
  else if DataSet.Name = 'cdsAttendance' then
    Result := ClubCode +tcAttendance +Result;
end;

procedure TdtmdlMain.SQLConnection1BeforeConnect(Sender: TObject);
begin
  (Sender as TSQLConnection).Params.Values['RoleName']      := 'RoleName';
  (Sender as TSQLConnection).Params.Values['ServerCharSet'] := '';
  (Sender as TSQLConnection).Params.Values['SQLDialect']    := '1';
  (Sender as TSQLConnection).Params.Values['ErrorResourceFile'] := '';
  (Sender as TSQLConnection).Params.Values['LocaleCode']    := '0000';
  (Sender as TSQLConnection).Params.Values['BlobSize=']     := '1';
  (Sender as TSQLConnection).Params.Values['CommitRetain']  := 'True';
  (Sender as TSQLConnection).Params.Values['WaitOnLocks']   := 'True';
  (Sender as TSQLConnection).Params.Values['Interbase TransIsolation'] := 'ReadCommited';
  (Sender as TSQLConnection).Params.Values['Trim Char']     := 'False';
  (Sender as TSQLConnection).Params.Values['User_Name']     := 'SYSDBA';
  (Sender as TSQLConnection).Params.Values['Password']      := 'masterkey';
end;

procedure TdtmdlMain.DataModuleDestroy(Sender: TObject);
begin
  try
    SQLConnection1.Close;
    Sleep(100);
  except
    on e:Exception do begin
      ShowMessage(e.message);
    end;
  end;
end;


procedure TdtmdlMain.DataModuleCreate(Sender: TObject);
var
  OK : Boolean;
  Str : string;
begin
  SQLConnection1.Close;
  
  OK := False;
  while not OK do begin
    InputQuery('Select Site','Please enter site number',Str);
    OK := Length(Str) = 3;
  end;
  ClubCode := Uppercase(Str);
  cdsClubInfo.FileName := ExtractFilePath(Application.ExeName)+'Club'+ClubCode+'Info.xml';

  if FileExists(cdsClubInfo.FileName) then begin
    cdsClubInfo.LoadFromFile(cdsClubInfo.FileName);
    SQLConnection1.Params.Values['Database'] := cdsClubInfo.FieldByName('DATABASE').AsString;
    try
      SQLConnection1.Open;
    except
      on e:exception do
        MessageDlg('Unable to connect to database. Please check settings'+#13+e.message, mtInformation, [mbOK], 0);
    end;
  end else begin
    MessageDlg('Creating new club. Please set database', mtInformation, [mbOK], 0);
    try
      cdsClubInfo.CreateDataSet;
      cdsClubInfo.Insert;
      cdsClubInfo.FieldByName('CLUB_CODE').AsString := ClubCode;
      cdsClubInfo.Post;
    except
      on e:exception do
        MessageDlg('Unable to create new config file'+#13+e.message, mtInformation, [mbOK], 0);
    end;
  end;
end;

procedure TdtmdlMain.SaveClubInfo(DataSet: TDataSet);
begin
  cdsClubInfo.MergeChangeLog;
  cdsClubInfo.SaveToFile('',dfXML);
end;

procedure TdtmdlMain.cdsReconcileError(
  DataSet: TCustomClientDataSet; E: EReconcileError;
  UpdateKind: TUpdateKind; var Action: TReconcileAction);
begin
  Action := HandleReconcileError(DataSet,UpdateKind,E);
end;

procedure TdtmdlMain.dsMembersDataChange(Sender: TObject; Field: TField);
begin
  cdsAttendance.Filter := 'MEMBER_UID='''+cdsMembers.FieldByName('UID').AsString+'''';
  cdsAttendance.Filtered := True;
end;

end.
